-- player.lua
-- @author Gerard Purra Teixido <gerard.purra@gmail.com>

-- Player = Entity representing the player
-- @field int x = X coordinate
-- @field int y = Y coordinate
-- @field int dx = Direction on X axis (-1, 0. 1)
-- @field int dy = Direction on Y axis (-1, 0. 1)
-- @field int speed = Movement speed (pixels/second)
local Player = {
  -- position
  x = 0,
  y = 0,
  -- direction
  dx = 0,
  dy = 0,
  -- facing
  facing = 0,
  -- dimensions
  w = 0,
  h = 0,
  -- speed
  current_speed = 0,
  walk_speed = 80,
  run_speed = 200,
  -- run / stamina
  is_running = false,
  stamina = 100,
  has_stamina = true,
  is_stamina_full = true,
  -- state
  current_state = 0,
  -- collider
  collider = nil,
  pushing_top = false,
  pushing_bottom = false,
  pushing_left = false,
  pushing_right = false,
  -- events
  trigger = nil,
  messenger = nil,
  -- weapon
  weapon = nil,
  -- timer
  time = 0
}

local player_mt = { __index = Player }

function Player.new(data)
  local instance = data
  return setmetatable(instance, player_mt)
end

function Player:update(dt)
  -- Calculate velocity
  local speed = 0
  if self.is_running and self.has_stamina then
    speed = self.run_speed
  else
    speed = self.walk_speed
  end
  self.current_speed = speed
  local velocity = speed * dt
  local dx, dy = self.dx, self.dy
  if dx ~= 0 and dy ~= 0 then
    dx, dy = Vector.normalize(dx, dy)
  end

  -- IDDLE
  if self.current_state == Globals.PLAYER_STATE_IDDLE then
    if dx ~= 0 or dy ~= 0 then
      self.current_state = Globals.PLAYER_STATE_WALK
      return
    end

    -- Recharge stamina
    if not self.is_stamina_full then
      self.stamina = math.min(self.stamina + 10 * dt, 100)
      if self.stamina == 100 then
        self.is_stamina_full = true
      elseif self.stamina > 20 then
        self.has_stamina = true
      end
    end

    return
  end
  -- WALK
  if self.current_state == Globals.PLAYER_STATE_WALK then
    if dx == 0 and dy == 0 then
      self.current_state = Globals.PLAYER_STATE_IDDLE
      return
    end

    if self.is_running and self.has_stamina then
      self.stamina = math.max(self.stamina - 10 * dt, 0)
      self.is_stamina_full = false
      if self.stamina == 0 then
        self.has_stamina = false
      end
    elseif not self.is_stamina_full then
      self.stamina = math.min(self.stamina + 10 * dt,100)
      if self.stamina == 100 then
        self.is_stamina_full = true
      elseif self.stamina > 20 then
        self.has_stamina = true
      end
    end
    self:move(dx * velocity, dy * velocity)
    return
  end
  -- ATTACK
  if self.current_state == Globals.PLAYER_STATE_ATTACK then
    self.weapon:update(dt)
    return
  end
  -- REBOUND
  if self.current_state == Globals.PLAYER_STATE_REBOUND then
    self.time = self.time + dt
    if self.time > 1 then
      self.time = 0
      self.current_state = Globals.PLAYER_STATE_IDDLE
    end

    return
  end
end

function Player:draw()
end

function Player:move(dx, dy)
  self.x, self.y = self.x + dx, self.y + dy
  self.collider:move(dx, dy)
  self.trigger:move(dx, dy)
end

function Player:attack()
  if not self.weapon.has_finished then
    return
  end

  self.current_state = Globals.PLAYER_STATE_ATTACK
  self.weapon:reset()
  -- x and y = center, a = angle of rotation, rx and ry = center of rotation
  local x, y, a, rx, ry
  if self.facing == Globals.ENTITY_FACE_UP then
    x = self.x
    y = self.y - self.weapon.w / 2
    a = -math.pi / 2
    rx = self.x + self.w
    ry = self.y + self.h / 2
  elseif self.facing == Globals.ENTITY_FACE_DOWN then
    x = self.x + self.w
    y = self.y + self.h + self.weapon.w / 2
    a = -math.pi / 2
    rx = self.x
    ry = self.y + self.h / 2
  elseif self.facing == Globals.ENTITY_FACE_LEFT then
    x = self.x - self.weapon.w / 2
    y = self.y + self.h
    -- a = math.pi / 1.5
    a = 0
    rx = self.x + self.w / 2
    ry = self.y
  else -- Globals.ENTITY_FACE_RIGHT
    x = self.x + self.w + self.weapon.w / 2
    y = self.y
    a = 0
    rx = self.x + self.w / 2
    ry = self.y + self.h
  end
  self.weapon:rotate(a)
  self.weapon:moveTo(x, y)
  self.weapon.rx, self.weapon.ry = rx, ry
end

function Player:center()
  return self.collider:center()
end

function Player:draw_debug()
  love.graphics.setColor(0, 255, 0, 255)
  love.graphics.print('CURRENT_STATE: ' .. self.current_state, 500, 50)
  love.graphics.print('FACING: ' .. self.facing, 500, 70)
  love.graphics.print('STAMINA: ' .. self.stamina, 500, 90)
  love.graphics.print('CURRENT_SPEED: ' .. self.current_speed, 500, 110)
end

return Player;

