-- level.lua Gamestate - Level
-- @author Gerard Purra Teixido <gerard.purra@gmail.com>

-- Globals game constants
Globals = require 'src/globals'

-- Simple Tiled Implementation
local STI = require 'lib/sti'
Map = nil

-- Jumper
local jumperGrid = require 'lib.jumper.grid'
local jumperPathfinder = require 'lib.jumper.pathfinder'
Pathfinder = nil

-- Collisions
local HC = require 'lib/hc'
CollisionManager = require 'src/utils/collision_manager'
TriggerManager = require 'src/utils/trigger_manager'

-- Camera
Camera = require 'lib/hump/camera'

-- Factories
local EntityFactory = require 'src/entities/factory'
local EnemyFactory = require 'src/entities/enemies/factory'
local WeaponFactory = require 'src/entities/weapons/factory'

local level = {
  camera = nil,
  player = nil,
  enemies = nil,
  blocks = nil
}

function level:init()
  -- INIT COLLISION AND TRIGGER MANAGERS
  CollisionManager:init(HC.new(200))
  TriggerManager:init(HC.new(200))

  -- LOAD MAP
  Map = STI('assets/maps/test.lua')

  -- Create a matrix representing walkable map area
  local jumper_map = {}

  local not_walkable = 0
  local walkable = 1

  local pathfinding_layer = Map.layers['pathfinding']
  local f = love.filesystem.newFile('map.txt')
  f:open('w')
  local map_string = ''
  for y, row in ipairs(pathfinding_layer.data) do
    jumper_map[y] = {}
    f:write('[ ')
    for x, column in ipairs(row) do
      jumper_map[y][x] = not_walkable
      if column.id == walkable then
        jumper_map[y][x] = walkable
      end
      f:write(jumper_map[y][x]) 
      if x ~= pathfinding_layer.width then
        f:write(', ')
      end
    end
    f:write(' ]\r\n')
  end
  f:close()

  -- INIT PATHFINDER
  local grid = jumperGrid(jumper_map)
  Pathfinder = jumperPathfinder(grid, 'JPS', walkable)

  -- INIT BLOCKS
  local blocks_layer = Map.layers['blocks']
  self.blocks = {}
  for i, obj in ipairs(blocks_layer.objects) do
    table.insert( 
      self.blocks,
      EntityFactory.new(
        'block',
        {
          x = obj.x, y = obj.y, w = obj.width, h = obj.height
        }
      )
    )
  end
  -- INIT PLAYER
  self.player = EntityFactory.new(
    'player',
    {
      x = 50, y = 50, w = 50, h = 50,
      current_state = Globals.PLAYER_STATE_IDDLE
    }
  )
  -- SET WEAPON
  self.player.weapon = WeaponFactory.new( 'sword', self.player )

  -- INIT ENEMIES
  self.enemies = {}
  self.enemies[#self.enemies + 1] = EnemyFactory.new(
    'slug',
    {
      x = 400, y = 400, w = 50, h = 50,
      current_state = Globals.ENEMY_STATE_STANDBY,
    }
  )

  -- SET CAMERA
  self.camera = Camera(self.player:center())
  self.camera:zoomTo(1)
end

-- Called every time when entering the state.
function level:enter()
  Gamestate.registerEvents()
end

-- Called every time when leaving the state.
function level:leave()

end

function level:update(dt)
  -- ENTITIES
  self.player:update(dt)
  for _, enemy in pairs(self.enemies) do
    enemy:update(dt)
  end
  -- COLLISION MANAGER
  CollisionManager:update()
  -- TRIGGER MANAGER
  TriggerManager:update()

  -- CAMERA
  self.camera:lookAt(self.player:center())
  self.camera.x = self.camera.x + AspectRatio.camera.offset_x
  self.camera.y = self.camera.y + AspectRatio.camera.offset_y

  -- CANVAS
  Canvas:renderTo(
    function()
      love.graphics.clear(0, 0, 0, 255)
      love.graphics.push()
      love.graphics.scale(AspectRatio.scale, AspectRatio.scale)
      self.camera:attach()
      CollisionManager:debug_draw()
      TriggerManager:debug_draw()
      self.camera:detach()
      self.enemies[1]:draw_debug()
      -- self.player:draw_debug()
      love.graphics.pop()
    end
  )
end

-- Draws content on screen
function level:draw()
  love.graphics.clear(255, 255, 255, 255)
  love.graphics.setColor( 255, 255, 255, 255 )

  -- Draw CANVAS
  love.graphics.setBlendMode("alpha", "premultiplied")
  love.graphics.draw(Canvas, AspectRatio.canvas.x, AspectRatio.canvas.y)
  love.graphics.setBlendMode("alpha")

  love.graphics.setColor( 0, 0, 255, 255 )
  love.graphics.print("LEVEL: Press Enter to go to OVER", 10, 10)
  love.graphics.print("LEVEL: Press P to go to PAUSE the game", 10, 30)

  -- Center lines
  local winWidth, winHeight = love.graphics.getWidth(), love.graphics.getHeight()
  love.graphics.line(winWidth / 2, 0, winWidth / 2, winHeight)
  love.graphics.line(0, winHeight / 2, winWidth, winHeight / 2)
end

function level:keyreleased(key, code)
  if key == 'return' then
    Gamestate.switch(gamestates.over)
  elseif key == 'p' or key == 'P' then
    return Gamestate.push(gamestates.pause)
  end

  if key == 'up' then
    if not love.keyboard.isDown('down') then
      self.player.dy = 0
    else
      self.player.facing = Globals.ENTITY_FACE_DOWN
      self.player.dy = 1
    end
  end
  if key == 'down' then
    if not love.keyboard.isDown('up') then
      self.player.dy = 0
    else
      self.player.facing = Globals.ENTITY_FACE_UP
      self.player.dy = -1
    end
  end
  if key == 'left' then
    if not love.keyboard.isDown('right') then
      self.player.dx = 0
    else
      self.player.facing = Globals.ENTITY_FACE_RIGHT
      self.player.dx = 1
    end
  end
  if key == 'right' then
    if not love.keyboard.isDown('left') then
      self.player.dx = 0
    else
      self.player.facing = Globals.ENTITY_FACE_LEFT
      self.player.dx = -1
    end
  end

  if key == 'z' then
    self.player:attack()
  end
  if key == 'x' then
    -- self.player:action_x()
  end
  if key == 'lshift' then
    self.player.is_running = false
  end
end

function level:keypressed(key, code)
  if key == 'up' then
    self.player.dy = -1
    if self.player.facing == Globals.ENTITY_FACE_DOWN then
      self.player.facing = Globals.ENTITY_FACE_UP
    elseif not love.keyboard.isDown('left') and not love.keyboard.isDown('right') then
      self.player.facing = Globals.ENTITY_FACE_UP
    end
  end
  if key == 'down' then
    self.player.dy = 1
    if self.player.facing == Globals.ENTITY_FACE_UP then
      self.player.facing = Globals.ENTITY_FACE_DOWN
    elseif not love.keyboard.isDown('left') and not love.keyboard.isDown('right') then
      self.player.facing = Globals.ENTITY_FACE_DOWN
    end
  end
  if key == 'left' then
    self.player.dx = -1
    if self.player.facing == Globals.ENTITY_FACE_RIGHT then
      self.player.facing = Globals.ENTITY_FACE_LEFT
    elseif not love.keyboard.isDown('up') and not love.keyboard.isDown('down') then
      self.player.facing = Globals.ENTITY_FACE_LEFT
    end
  end
  if key == 'right' then
    self.player.dx = 1
    if self.player.facing == Globals.ENTITY_FACE_LEFT then
      self.player.facing = Globals.ENTITY_FACE_RIGHT
    elseif not love.keyboard.isDown('up') and not love.keyboard.isDown('down') then
      self.player.facing = Globals.ENTITY_FACE_RIGHT
    end
  end
  if key == 'lshift' then
    self.player.is_running = true
  end
end

return level
