-- CollisionManager.lua
-- @author Gerard Purra Teixido <gerard.purra@gmail.com>

local CollisionManager = {
  hc = nil,
  dynamics = nil,
  kinematics = nil,
  statics = nil
}

function CollisionManager:init(hc)
  self.hc = hc
  self.dynamics = {}
  self.kinematics = {}
  self.statics = {}
end

-- CollisionManager:add
-- @param collider - instance of Collider
function CollisionManager:add(collider)
  if collider.mode == Globals.COLLIDER_DYNAMIC then
    table.insert(self.dynamics, collider)
  elseif collider.mode == Globals.COLLIDER_KINEMATIC then
    table.insert(self.kinematics, collider)
  elseif collider.mode == Globals.COLLIDER_STATIC then
    table.insert(self.statics, collider)
  end
end

-- CollisionManager:update - Updates collider positions according to parents
-- @param dt - float Delta Time
function CollisionManager:update()
  -- Reset booleans for dynamic colliders
  for _, dyn in ipairs(self.dynamics) do
    local ent_d = dyn.entity
    ent_d.pushes_bottom, ent_d.pushes_top = false, false
    ent_d.pushing_left, ent_d.pushing_right = false, false
  end
  -- Resolve collision two times to handle dyn vs dyn acurrately
  self:resolveCollisions()
  self:resolveCollisions()
end

-- CollisionManager:remove - Removes collider instance
-- @param collider - Collider instance to be removed
function CollisionManager:remove(collider)
  self.hc:remove(collider)
  collider.remove = true
  if collider.mode == Globals.COLLIDER_DYNAMIC then
    for i, v in ipairs(self.dynamics) do
      if v.remove then
        table.remove(self.dynamics, i)
        return
      end
    end
    table.remove(self.dynamics, index)
  elseif collider.mode == Globals.COLLIDER_KINEMATIC then
    for i, v in ipairs(self.kinematics) do
      if v.remove then
        table.remove(self.kinematics, i)
      end
    end
    table.remove(self.kinematics, index)
  elseif collider.mode == Globals.COLLIDER_STATIC then
    for i, v in ipairs(self.statics) do
      if v.remove then
        table.remove(self.statics, i)
        return
      end
    end
  end
end

-- CollisionManager:clear - Clear all tables to reset colliders
function CollisionManager:clear()
  self.hc:resetHash(200)
  for k, v in pairs(self.dynamics) do
    self.dynamics[k] = nil
  end
  for k, v in pairs(self.kinematics) do
    self.kinematics[k] = nil
  end
  for k, v in pairs(self.statics) do
    self.statics[k] = nil
  end
end

function CollisionManager:resolveCollisions()
  for _, dyn in ipairs(self.dynamics) do
    if dyn.is_active then
      local collisions = self.hc:collisions(dyn)
      for other, sv in pairs(collisions) do
        if other.is_active
          and ( dyn.mask[other.group] or dyn.mask[Globals.MASK_ALL]) then
          if other.mode == Globals.COLLIDER_DYNAMIC then
            CollisionManager.resolveDynVsDyn(dyn, other, sv)
          else
            CollisionManager.resolveDynVsOther(dyn, other, sv)
          end
        end
      end
    end
  end
end


-- @param dyn - Entity of the dynamic collider
-- @param other - Entity of the other collider
function CollisionManager.resolveDynVsOther(dyn, other, sv)
  local entity = dyn.entity
  entity:move(sv.x, sv.y)

  local dyn_cx, dyn_cy = dyn:center()
  local other_cx, other_cy = other:center()

  if sv.y  ~= 0.0 then
    if other_cy < dyn_cy then
      entity.pushes_top = true
    else
      entity.pushes_bottom = true
    end
  end

  if sv.x ~= 0.0 then
    if other_cx > dyn_cx then
      entity.pushing_right = true
    else
      entity.pushing_left = true
    end
  end
end

function CollisionManager.resolveDynVsDyn(shape_a, shape_b, sv)
  local entity_a = shape_a.entity
  local entity_b = shape_b.entity

  entity_a:move(sv.x / 2, sv.y / 2)
  entity_b:move(-sv.x / 2,-sv.y / 2)
  
  -- if sv.x ~= 0.0 then
  --   if sv.x > 0 then -- gotta go left
  --     if entity_a.pushing_left then
  --       entity_b:move(-sv.x, -sv.y)
  --     else
  --       entity_a:move(sv.x, sv.y)
  --     end
  --   else -- gotta go right
  --     if entity_a.pushing_right then
  --       entity_b:move(-sv.x, -sv.y)
  --     else
  --       entity_a:move(sv.x, sv.y)
  --     end
  --   end
  -- else
  --   if sv.y < 0 then -- gotta go up
  --     if entity_a.pushes_top then
  --       entity_b:move(-sv.x, -sv.y)
  --     else
  --       entity_a:move(sv.x, sv.y)
  --     end
  --   else -- gotta go down
  --     if entity_a.pushes_bottom then
  --       entity_b:move(-sv.x, -sv.y)
  --     else
  --       entity_a:move(sv.x, sv.y)
  --     end
  --   end
  -- end
end

function CollisionManager:debug_draw()
  love.graphics.setColor(255, 0, 0, 255)
  for i, collider in ipairs(self.dynamics) do
    if collider.is_active then
      if collider.group == Globals.MASK_PLAYER then
        love.graphics.setColor(255, 255, 255, 255)
        collider:draw('line')
        love.graphics.setColor(255, 0, 0, 255)
      else
        collider:draw('line')
      end
    end
  end
  love.graphics.setColor(0, 0, 255, 255)
  for i, collider in ipairs(self.kinematics) do
    if collider.is_active then
      collider:draw('line')
    end
  end
  love.graphics.setColor(0, 255, 0, 255)
  for i, collider in ipairs(self.statics) do
    if collider.is_active then
      collider:draw('line')
    end
  end
  love.graphics.setColor(255, 255, 255, 255)
end

return CollisionManager
